/*
 * apple2fs driver for apple2 ucsd disk file system access
 * copyright (c)2003 GPL Oskar Schirmer (oskar@scara.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <asm/byteorder.h>

#define APPLE2FS_BLOCKSIZE_BITS	9
#define APPLE2FS_BLOCKSIZE	(1 << APPLE2FS_BLOCKSIZE_BITS)
#define APPLE2FS_ROOTDIR	2
#define APPLE2FS_DIRNLEN	7
#define APPLE2FS_FILENLEN	15
#define APPLE2FS_TEXTPHYSBEGIN	(2 * APPLE2FS_BLOCKSIZE)

#define APPLE2FS_FTYPE_DIR	0
#define APPLE2FS_FTYPE_CODE	2
#define APPLE2FS_FTYPE_TEXT	3
#define APPLE2FS_FTYPE_INFO	4
#define APPLE2FS_FTYPE_DATA	5
#define APPLE2FS_FTYPE_GRAF	6
#define APPLE2FS_FTYPE_FOTO	7
#define APPLE2FS_NR_FTYPE	8

#define APPLE2FS_DATE_DAY(d)	((d >> 4) & 0x1F)
#define APPLE2FS_DATE_MONTH(d)	(d & 0x0F)
#define APPLE2FS_DATE_YEAR(d)	((d >> 9) & 0x7F)

#define APPLE2FS_DE_FIRSTBLK		0
#define APPLE2FS_DE_FOLLOWBLK		2
#define APPLE2FS_DE_FILETYPE		4
#define APPLE2FS_DE_NLEN		6
#define APPLE2FS_DE_NAME		7
#define APPLE2FS_DE_NBLOCKS		14
#define APPLE2FS_DE_NFILES		16
#define APPLE2FS_DE_LOADTIME		18
#define APPLE2FS_DE_LASTBOOTDATE	20
#define APPLE2FS_DE_USEDBYTESINLASTBLK	22
#define APPLE2FS_DE_MODDATE		24
#define APPLE2FS_SIZEOF_DIRENTRY	26

struct apple2fs_direntry {
	__u16 firstblk;
	__u16 followblk;
	__u16 filetype;
	union {
		struct {
			__u8 dirnlen;
			__u8 dirname[APPLE2FS_DIRNLEN];
			__u16 nblocks;
			__u16 nfiles;
			__u16 loadtime;
			__u16 lastbootdate;
		} dir;
		struct {
			__u8 filenlen;
			__u8 filename[APPLE2FS_FILENLEN];
			__u16 usedbytesinlastblk;
			__u16 moddate;
		} file;
	} u;
};

#define APPLE2FS_ROOT_INO	2
#define APPLE2FS_TEXT_INO	1
#define APPLE2FS_INO_SHIFT	1
#define APPLE2FS_INO_BASE	1
#define APPLE2FS_INO_FILE(n)	((n + APPLE2FS_INO_BASE) << APPLE2FS_INO_SHIFT)
#define APPLE2FS_INO_FNDX(i)	((i >> APPLE2FS_INO_SHIFT) - APPLE2FS_INO_BASE)
#define APPLE2FS_INO_MASK	((1 << APPLE2FS_INO_SHIFT) - 1)

struct apple2fs_sb_info {
	int nblocks;
	int nfiles;
	int firstdatablk;
	unsigned char subdirlong[APPLE2FS_INO_MASK];
};
#define APPLE2FS_SBU_INFO(sb,field) \
	(((struct apple2fs_sb_info *)sb->u.generic_sbp)->field)

struct apple2fs_inode_info {
	int firstblock;
};
#define APPLE2FS_IPU_INFO(ip,field) \
	(((struct apple2fs_inode_info *)ip->u.generic_ip)->field)

struct apple2fs_file_info {
	loff_t phys_pos;
	loff_t last_fpos;
	int leftover_dle;
	int accum_null;
};

extern const char *const subdirnames[];

extern struct file_operations apple2fs_dir_operations;
extern struct file_operations apple2fs_file_operations;
extern struct inode_operations apple2fs_dir_inode_operations;

extern void apple2fs_read_inode (struct inode *inode);
extern void apple2fs_put_inode(struct inode *inode);
extern int apple2fs_getrootdirentry(struct super_block *sb, int filenr,
	struct apple2fs_direntry *dire);
extern int apple2fs_scantextfile(struct super_block *sb,
	struct apple2fs_file_info *pd,
	int firstblock, int count, char *buf);
